<?php

namespace Drupal\simple_group_invite\Entity\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Access\AccessResultNeutral;
use Drupal\Core\Access\AccessResultReasonInterface;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityHandlerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\simple_group\Access\SimpleGroupPermissions;
use Drupal\simple_group\Entity\SimpleGroup;
use Drupal\simple_group_invite\Entity\SimpleGroupInvite;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Access control handler for the Simple Group entity.
 */
class SimpleGroupInviteAccessControlHandler extends EntityAccessControlHandler implements EntityHandlerInterface {

  //use AutowireTrait;

  /**
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type): SimpleGroupInviteAccessControlHandler|static {
    return new static(
      $entity_type,
      $container->get('entity_field.manager'),
    );
  }

  public function __construct(EntityTypeInterface $entityType, EntityFieldManagerInterface $entity_field_manager) {
    parent::__construct($entityType);
    $this->entityFieldManager = $entity_field_manager;
  }


  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account): AccessResultReasonInterface|AccessResultNeutral|AccessResult|AccessResultAllowed|AccessResultInterface {
    /** @var SimpleGroupInvite $entity */

    switch ($operation) {
      case 'view':
      case 'update':
      case 'delete':
        return $this->useSimpleGroupPermissions($account, $entity);
      default:
        return AccessResult::neutral();
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL): AccessResultReasonInterface|AccessResult|AccessResultInterface {
    return $this->useSimpleGroupPermissions($account);
  }

  private function getGroupReferenceField(AccountInterface $user): ?string {
    $field_definitions = $this->entityFieldManager->getFieldDefinitions('user', $user->bundle());

    foreach ($field_definitions as $field_name => $field_definition) {
      // Check if the field is an entity reference field.
      if ($field_definition->getType() === 'entity_reference') {
        // Check if the field references the SimpleGroup entity type.
        if ($field_definition->getSetting('target_type') === 'simple_group') {
          return $field_name;
        }
      }
    }
    return null;
  }

  private function useSimpleGroupPermissions(AccountInterface $account, EntityInterface $entity = null): AccessResultNeutral|AccessResultAllowed {
    /** @var SimpleGroupInvite $entity */

    if ($account->hasPermission(SimpleGroupPermissions::VIEW_ALL_SIMPLE_GROUPS) or
      $account->hasPermission(SimpleGroupPermissions::MANAGE_SIMPLE_GROUPS) or
      $account->hasPermission(SimpleGroupPermissions::ADMINISTER_SIMPLE_GROUPS)
    ) {
      return AccessResult::allowed();
    }

    if ($account->hasPermission(SimpleGroupPermissions::VIEW_OWNED_SIMPLE_GROUPS)) {
      if (!empty($entity)) {
        $group = SimpleGroup::load($entity->get('group_id')->getString());
        if ($group instanceof SimpleGroup) {
          if ($account->id() == $group->getOwnerId()) {
            return AccessResult::allowed();
          }
        }
      }else{
        // We are creating a new entity so we should allow.
        return AccessResult::allowed();
      }
    }

    if ($account->hasPermission(SimpleGroupPermissions::VIEW_ASSIGNED_SIMPLE_GROUPS)) {
      if (!empty($entity)) {
        $user = User::load($account->id());
        $referenceField = $this->getGroupReferenceField($user, $entity);

        if ($referenceField) {
          $groups = $user->get($referenceField)->referencedEntities();
          foreach ($groups as $group) {
            if ($group->id() == $entity->get('group_id')->value) {
              return AccessResult::allowed();
            }
          }
        }
      }else{
        // We are going not going to let assignees send invites.
        return AccessResult::neutral();
      }
    }

    // If we make it this far, let's just not allow it.
    return AccessResult::neutral();
  }

}
