<?php

namespace Drupal\simple_group_invite\Entity\Form;

use Drupal\Component\Utility\EmailValidatorInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Access\AccessResultNeutral;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\simple_group\Entity\SimpleGroup;
use Drupal\simple_group_invite\Entity\SimpleGroupInvite;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SimpleGroupInviteAddForm extends EntityForm {

  /**
   * @var \Drupal\Core\Routing\RouteMatchInterface $routeMatch
   */
  protected $routeMatch;

  /**
   * @var \Drupal\Component\Utility\EmailValidatorInterface $emailValidator
   */
  protected EmailValidatorInterface $emailValidator;

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface $mailManager
   */
  protected MailManagerInterface $mailManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(RouteMatchInterface $route_match, EmailValidatorInterface $emailValidator, MailManagerInterface $mail_manager) {
    $this->routeMatch = $route_match;
    $this->emailValidator = $emailValidator;
    $this->mailManager = $mail_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('current_route_match'),
      $container->get('email.validator'),
      $container->get('plugin.manager.mail'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_group_invite_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    // Get the SimpleGroup ID from the route
    $group = $this->routeMatch->getParameter('simple_group_id');

    // Add an email field
    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Email address'),
      '#required' => TRUE,
    ];

    // Add the group ID as a hidden field
    $form['group_id'] = [
      '#type' => 'hidden',
      '#value' => $group->id(),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send Invite'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // Basic email validation, or more advanced as needed.
    if (!$this->emailValidator->isValid($form_state->getValue('email'))) {
      $form_state->setErrorByName('email', $this->t('Please enter a valid email address.'));
    }
  }

  /**
   * {@inheritdoc}
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $email = $form_state->getValue('email');
    $group_id = $form_state->getValue('group_id');
    $currentUser = User::load($this->currentUser()->id());

    // Check if the user with this email already exists.
    $user = user_load_by_mail($email);

    $invite = SimpleGroupInvite::create([
      'email' => $email,
      'group_id' => $group_id,
      'status' => 'pending',
    ]);
    $invite->save();

    if (!$user) {
      // User exists, create a pending invite.

      // You'd use a mail system to send this registration invite.
      $from = $this->configFactory()->get('system.site')->get('mail');
      // Define the email parameters
      // TODO: Put this email into the UI system.
      $params = [
        'subject' => t('You have been invited to join RAMW as part of a group.'),
        'message' => t(
          '
          <p>Hello, you have been invited to join @name group on RAMW.</p>
          <p>You can use this link to register: @regLink</p>
          <p>Once you have registered, you will automatically be assigned to the group
          and can begin helping @name with their business.</p>
          ',
          [
            '@name' => $currentUser->get('field_user_first_name')->value . " " . $currentUser->get('field_user_last_name')->value,
            '@regLink' => Url::fromRoute('user.register')->toString(),
          ]
        ),
        'headers' => [
          'Content-Type' => 'text/html; charset=UTF-8',
        ]
      ];

      $this->mailManager->mail('simple_group_invite', 'invite', $email, 'en', $params, $from, TRUE);
    }

    $this->messenger()->addMessage($this->t('Invite sent to %email', [
      '%email' => $email,
    ]));

    $url = Url::fromRoute('entity.simple_group.canonical', ['simple_group' => $group_id]);
    $form_state->setRedirectUrl($url);
  }
}
