<?php

namespace Drupal\simple_group_invite\Form;

use Drupal\Component\Utility\EmailValidatorInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Access\AccessResultNeutral;
use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\simple_group\Entity\SimpleGroup;
use Drupal\simple_group_invite\Entity\SimpleGroupInvite;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SimpleGroupInviteForm extends FormBase {

  /**
   * @var \Drupal\Core\Routing\RouteMatchInterface $routeMatch
   */
  protected $routeMatch;

  /**
   * @var \Drupal\Component\Utility\EmailValidatorInterface $emailValidator
   */
  protected EmailValidatorInterface $emailValidator;

  /**
   * {@inheritdoc}
   */
  public function __construct(RouteMatchInterface $route_match, EmailValidatorInterface $emailValidator) {
    $this->routeMatch = $route_match;
    $this->emailValidator = $emailValidator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): SimpleGroupInviteForm|static {
    return new static(
      $container->get('current_route_match'),
      $container->get('email.validator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_group_invite_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    // Get the SimpleGroup ID from the route
    $group_id = $this->routeMatch->getParameter('group_id');

    // Add an email field
    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Email address'),
      '#required' => TRUE,
    ];

    // Add the group ID as a hidden field
    $form['simple_group_id'] = [
      '#type' => 'hidden',
      '#value' => $group_id,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send Invite'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // Basic email validation, or more advanced as needed.
    if (!$this->emailValidator->isValid($form_state->getValue('email'))) {
      $form_state->setErrorByName('email', $this->t('Please enter a valid email address.'));
    }
  }

  /**
   * {@inheritdoc}
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $email = $form_state->getValue('email');
    $group_id = $form_state->getValue('simple_group_id');

    // Check if the user with this email already exists.
    $user = user_load_by_mail($email);

    if ($user) {
      // User exists, create a pending invite.
      $invite = SimpleGroupInvite::create([
        'email' => $email,
        'simple_group_id' => $group_id,
        'status' => 'pending',
      ]);
      $invite->save();

      // Send the accept invitation link.
      $accept_link = Url::fromRoute('simple_group.invite_accept', ['invite_id' => $invite->id()], ['absolute' => TRUE]);
      // You'd use a mail system to send this invite.
    }
    else {
      // User does not exist, create a registration invite.
      $invite = SimpleGroupInvite::create([
        'email' => $email,
        'simple_group_id' => $group_id,
        'status' => 'pending',
      ]);
      $invite->save();

      // Send a registration link with the invite accept destination.
      $register_link = Url::fromRoute('user.register', [], [
        'absolute' => TRUE,
        'query' => ['destination' => 'simple_group/invite/accept/' . $invite->id()],
      ]);
      // You'd use a mail system to send this registration invite.
    }

    $this->messenger()->addMessage($this->t('Invite sent to %email for group %group_id', [
      '%email' => $email,
      '%group_id' => $group_id,
    ]));
  }

  /**
   * Custom access check.
   */
  public static function access(AccountInterface $account, SimpleGroup $group): AccessResultNeutral|AccessResult|AccessResultAllowed {
    // Assuming the SimpleGroup entity has an owner property or method.
    $group_owner_id = $group->getOwnerId();

    // Check if the logged-in user is the owner of the group.
    return AccessResult::allowedIf($group_owner_id == $account->id());
  }
}
