<?php

namespace Drupal\simple_group\Entity\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Access\AccessResultNeutral;
use Drupal\Core\Access\AccessResultReasonInterface;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityHandlerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\simple_group\Access\SimpleGroupPermissions;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Access control handler for the Simple Group entity.
 */
class SimpleGroupAccessControlHandler extends EntityAccessControlHandler implements EntityHandlerInterface {

  //use AutowireTrait;

  /**
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type): SimpleGroupAccessControlHandler|static {
    return new static(
      $entity_type,
      $container->get('entity_field.manager'),
    );
  }

  public function __construct(EntityTypeInterface $entityType, EntityFieldManagerInterface $entity_field_manager) {
    parent::__construct($entityType);
    $this->entityFieldManager = $entity_field_manager;
  }


    /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account): AccessResultReasonInterface|AccessResultNeutral|AccessResult|AccessResultAllowed|AccessResultInterface {
    switch ($operation) {
      case 'view':
        if ($account->hasPermission(SimpleGroupPermissions::VIEW_ALL_SIMPLE_GROUPS) or
          $account->hasPermission(SimpleGroupPermissions::MANAGE_SIMPLE_GROUPS) or
          $account->hasPermission(SimpleGroupPermissions::ADMINISTER_SIMPLE_GROUPS)
        ) {
          return AccessResult::allowed();
        }

        if ($account->hasPermission(SimpleGroupPermissions::VIEW_OWNED_SIMPLE_GROUPS)) {
          if ($account->id() == $entity->getOwnerId()) {
            return AccessResult::allowed();
          }
        }

        if ($account->hasPermission(SimpleGroupPermissions::VIEW_ASSIGNED_SIMPLE_GROUPS)) {
          $user = User::load($account->id());
          $referenceField = null;
          $field_definitions = $this->entityFieldManager->getFieldDefinitions('user', $user->bundle());

          $test = [];
          foreach ($field_definitions as $field_name => $field_definition) {
            // Check if the field is an entity reference field.
            if ($field_definition->getType() === 'entity_reference') {
              // Check if the field references the SimpleGroup entity type.
              if ($field_definition->getSetting('target_type') === $entity->getEntityTypeId()) {
                $referenceField = $field_name;
                break;
              }
            }
          }

          if ($referenceField) {
            $groups = $user->get($referenceField)->referencedEntities();
            foreach ($groups as $group) {
              if ($group->id() == $entity->id()) {
                return AccessResult::allowed();
              }
            }
          }
        }
        break;
      case 'update':
        if ($account->hasPermission(SimpleGroupPermissions::MANAGE_SIMPLE_GROUPS) or
          $account->hasPermission(SimpleGroupPermissions::ADMINISTER_SIMPLE_GROUPS)
        ) {
          return AccessResult::allowed();
        }
        break;
      case 'delete':
        if ($account->hasPermission(SimpleGroupPermissions::ADMINISTER_SIMPLE_GROUPS)) {
          return AccessResult::allowed();
        }
        break;
    }
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL): AccessResultReasonInterface|AccessResult|AccessResultInterface {
    return AccessResult::allowedIfHasPermission($account, SimpleGroupPermissions::ADMINISTER_SIMPLE_GROUPS);
  }

}
