<?php

namespace Drupal\simple_group\Entity\Handler;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\simple_group\Entity\SimpleGroup;
use Drupal\simple_group\Entity\SimpleGroupInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Routing\RouterInterface;

/**
 * Provides a list controller for simple_group entities.
 *
 * @ingroup simple_group
 */
class SimpleGroupListBuilder extends EntityListBuilder {

  /**
   * The redirect destination service.
   *
   * @var \Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $redirectDestination;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The router.
   *
   * @var \Symfony\Component\Routing\RouterInterface
   */
  protected $router;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a new GroupListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage class.
   * @param \Drupal\Core\Routing\RedirectDestinationInterface $redirect_destination
   *   The redirect destination service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Symfony\Component\Routing\RouterInterface $router
   *   The router.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *    The request stack service.
   * @param \Drupal\Core\Database\Connection $database
   *    The database connection.
 */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage, RedirectDestinationInterface $redirect_destination, AccountInterface $current_user, ModuleHandlerInterface $module_handler, RouterInterface $router, RequestStack $request_stack, Connection $database) {
    parent::__construct($entity_type, $storage);
    $this->redirectDestination = $redirect_destination;
    $this->currentUser = $current_user;
    $this->moduleHandler = $module_handler;
    $this->router = $router;
    $this->requestStack = $request_stack;
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage($entity_type->id()),
      $container->get('redirect.destination'),
      $container->get('current_user'),
      $container->get('module_handler'),
      $container->get('router.no_access_checks'),
      $container->get('request_stack'),
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header = [
      'group_id' => [
        'data' => $this->t('Group ID'),
        'specifier' => 'id',
        'field' => 'id',
        'sortable' => TRUE,
      ],
      'group_name' => [
        'data' => $this->t('Group Name'),
        'specifier' => 'label',
        'field' => 'label',
        'sortable' => TRUE,
      ],
      'type' => [
        'data' => $this->t('Type'),
        'specifier' => 'type',
        'field' => 'type',
      ],
      'published' => [
        'data' => $this->t('Status'),
        'specifier' => 'published',
        'field' => 'published',
      ],
      'owner' => [
        'data' => $this->t('Owner'),
      ],
    ];
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    assert($entity instanceof SimpleGroupInterface);
    /** @var SimpleGroup $entity */
    $row['id'] = $entity->id();
    // EntityListBuilder sets the table rows using the #rows property, so we
    // need to add the render array using the 'data' key.
    $row['label']['data'] = $entity->toLink()->toRenderable();
    $row['type'] = is_null($entity->getSimpleGroupType())
      ? ''
      : $entity->getSimpleGroupType()->label();
    $row['status'] = $entity->isPublished()
      ? $this->t('Active')
      : $this->t('Inactive');
    if ($entity->getOwner()) {
      $row['uid'] = $entity->getOwner()->label();
    }else {
      $row['uid'] = '';
    }
    $row['operations']['data'] = $this->buildOperations($entity);
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build = [];

    // Get the current request object.
    $current_request = $this->requestStack->getCurrentRequest();

    // Get any search filters provided by the user through the query.
    $label_filter = $current_request->query->get('label');

    // Add a filter form (exposed filter text input).
    $form = [
      '#type' => 'form',
      '#method' => 'get',
      'label' => [
        '#type' => 'textfield',
        '#name' => 'label',
        '#title' => $this->t('Search by Group Name'),
        '#default_value' => $label_filter,
      ],
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Filter'),
      ],
    ];
    $build['filter_form'] = $form;

    $build['table'] = parent::render();
    $build['table']['#empty'] = $this->t('There are no groups yet.');
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function load(): array {
    // Build a custom query to load filtered results.
    $query = $this->getStorage()->getQuery();

    // Get the current request object.
    $current_request = $this->requestStack->getCurrentRequest();

    // Get the label filter from the request query parameters (if any).
    $label_filter = $current_request->query->get('label');
    $query_order = $current_request->query->get('order');
    $query_sort = $current_request->query->get('sort');
    if (!empty($label_filter)) {
      // Use the database connection's escapeLike() for safe LIKE filtering.
      $escaped_filter = $this->database->escapeLike($label_filter);
      // Add a condition to filter by the label using a case-insensitive search.
      $query->condition('label', '%' . $escaped_filter . '%', 'LIKE');
    }

    $header = $this->buildHeader();
    $query->tableSort($header);

    // Execute the query to get the entity IDs.
    $entity_ids = $query->accessCheck()->execute();

    // Load the filtered entity objects.
    return $this->getStorage()->loadMultiple($entity_ids);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntityIds(): int|array {
    $query = $this->getStorage()->getQuery();

    // Add a simple table sort by header, see ::buildHeader().
    $header = $this->buildHeader();
    $query->tableSort($header);

    // Only add the pager if a limit is specified.
    if ($this->limit) {
      $query->pager($this->limit);
    }

    return $query->accessCheck()->execute();
  }
}
