<?php

namespace Drupal\simple_group\Entity;

use Drupal\Core\Entity\Annotation\ContentEntityType;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\user\EntityOwnerTrait;
use Drupal\user\StatusItem;

/**
 * Defines the Group entity.
 *
 * @ContentEntityType(
 *   id = "simple_group",
 *   label = @Translation("Simple Group"),
 *   label_singular = @Translation("simple group"),
 *   label_plural = @Translation("simple groups"),
 *   label_collection = @Translation("Simple Groups"),
 *   label_count = @PluralTranslation(
 *     singular = "@count simple group",
 *     plural = "@count simple groups"
 *   ),
 *   bundle_label = @Translation("Simple Group Type"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\simple_group\Entity\Handler\SimpleGroupListBuilder",
 *     "access" = "Drupal\simple_group\Entity\Access\SimpleGroupAccessControlHandler",
 *     "form" = {
 *       "default" = "Drupal\simple_group\Entity\Form\SimpleGroupForm",
 *       "add" = "Drupal\simple_group\Entity\Form\SimpleGroupForm",
 *       "edit" = "Drupal\simple_group\Entity\Form\SimpleGroupForm",
 *       "delete" = "Drupal\simple_group\Entity\Form\SimpleGroupDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\simple_group\Entity\Routing\SimpleGroupRouteProvider",
 *     },
 *   },
 *   base_table = "simple_group",
 *   data_table = "simple_group_field_data",
 *   translatable = TRUE,
 *   field_ui_base_route = "entity.simple_group_type.edit_form",
 *   admin_permission = "administer nodes",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *     "owner" = "uid",
 *     "langcode" = "langcode",
 *     "label" = "label",
 *     "published" = "status",
 *     "bundle" = "type",
 *   },
 *   bundle_entity_type = "simple_group_type",
 *   links = {
 *     "add-form" = "/simple-group/add/{simple_group_type}",
 *     "add-page" = "/simple-group/add",
 *     "canonical" = "/simple-group/{simple_group}",
 *     "collection" = "/admin/simple-group",
 *     "edit-form" = "/simple-group/{simple_group}/edit",
 *     "delete-form" = "/simple-group/{simple_group}/delete",
 *   },
 * )
 */
class SimpleGroup extends ContentEntityBase implements SimpleGroupInterface {

  use EntityChangedTrait;
  use EntityPublishedTrait;
  use EntityOwnerTrait;

  public function getSimpleGroupType(): SimpleGroupType {
    return $this->type->entity;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);
    $fields += static::publishedBaseFieldDefinitions($entity_type);
    $fields += static::ownerBaseFieldDefinitions($entity_type);

    $fields['label'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Group Name'))
      ->setRequired(TRUE)
      ->setTranslatable(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->addConstraint('UniqueField', [])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayConfigurable('form', TRUE);

    // Add the bundle (type) field.
    $fields['type'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Simple Group Type'))
      ->setDescription(t('The type of the Simple Group entity.'))
      ->setSetting('target_type', 'simple_group_type')
      ->setSetting('handler', 'default')
      ->setTranslatable(FALSE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'entity_reference_label',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 1,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setRequired(TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created on'))
      ->setDescription(t('The time that the group was created.'))
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'region' => 'hidden',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed on'))
      ->setDescription(t('The time that the group was last edited.'))
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'region' => 'hidden',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);

    if (\Drupal::moduleHandler()->moduleExists('path')) {
      $fields['path'] = BaseFieldDefinition::create('path')
        ->setLabel(t('URL alias'))
        ->setTranslatable(TRUE)
        ->setDisplayOptions('form', [
          'type' => 'path',
          'weight' => 30,
        ])
        ->setDisplayConfigurable('form', TRUE)
        ->setComputed(TRUE);
    }
    return $fields;
  }
}
